package com.jarsilio.android.common.extensions

import android.content.Context
import android.net.Uri
import androidx.core.content.FileProvider
import timber.log.Timber
import java.io.File

val Context.actualPackageName: String?
    get() = applicationContext.javaClass.`package`?.name

val Context.applicationId: String?
    get() = packageName

val Context.flavor: String?
    get() = getBuildConfigValue(actualPackageName, "FLAVOR") as String?

val Context.appName: String?
    get() {
        val applicationInfo = applicationContext.applicationInfo
        val stringId = applicationInfo.labelRes
        return if (stringId == 0) {
            applicationInfo.nonLocalizedLabel.toString()
        } else {
            applicationContext.getString(stringId)
        }
    }

val Context.shortAppName: String?
    get() = actualPackageName?.substringAfterLast('.')

val Context.fileProviderAuthority: String
    get() = "$applicationId.common.fileprovider"

val Context.defaultEmail: String
    get() = "juam+$shortAppName@posteo.net"

/* Logging */
val Context.logsDir: File
    get() = File(cacheDir.toString(), "logs") // like in filepaths.xml

val Context.mergedLogFile: File
    get() = File(logsDir, "$shortAppName.log")
val Context.mergedLogFileUri: Uri
    get() = FileProvider.getUriForFile(this, fileProviderAuthority, mergedLogFile)

val Context.logFiles: Array<File>
    get() = arrayOf(
        File(logsDir, "$shortAppName.log.1"),
        File(logsDir, "$shortAppName.log.2")
    )
val Context.logFile: File
    get() = logFiles[0]
val Context.rotatedLogFile: File
    get() = logFiles[1]

/**
 * Gets a field from the project's BuildConfig. This is useful when, for example, flavors
 * are used at the project level to set custom fields.
 * @param fieldName The name of the field-to-access
 * @return The value of the field, or `null` if the field is not found.
 */
private fun getBuildConfigValue(packageName: String?, fieldName: String): Any? {
    val buildConfigClassName = "$packageName.BuildConfig"
    try {
        val clazz = Class.forName(buildConfigClassName)
        val field = clazz.getField(fieldName)
        return field.get(null)
    } catch (e: ClassNotFoundException) {
        Timber.e(e, "Failed to get $buildConfigClassName.$fieldName")
    } catch (e: NoSuchFieldException) {
        Timber.e(e, "Failed to get $buildConfigClassName.$fieldName")
    } catch (e: IllegalAccessException) {
        Timber.e(e, "Failed to get $buildConfigClassName.$fieldName")
    }

    return null
}